<?php
namespace App\Controller;

use Cake\Core\Configure;
use Cake\Network\Exception\ForbiddenException;
use Cake\Network\Exception\NotFoundException;
use Cake\View\Exception\MissingTemplateException;

class RadioController extends AppController {

	public function initialize() {
		parent::initialize();
		$this->loadComponent('RequestHandler');
	}

	public function transmit($x = 0.0, $y = 0.0, $z = 0.0) {
		$this->loadModel('Messages');

		// Get and clean coordinates
		$coordinates = [
			'x' => floatval(str_replace(',', '.', $x)),
			'y' => floatval(str_replace(',', '.', $y)),
			'z' => floatval(str_replace(',', '.', $z))
		];

		$messages = [];
		$hints = [];

		// Check coordinates
		foreach (['x', 'y', 'z'] as $dimension) {
			if ($coordinates[$dimension] > 180.0) {
				$coordinates[$dimension] = 180.0;
				$messages[] = $this->Messages->createDynamicMessage('Dimension ' . strtoupper($dimension) . ' superspace prohibited.', ['warning']);
			}

			if ($coordinates[$dimension] < -180.0) {
				$coordinates[$dimension] = -180.0;
				$messages[] = $this->Messages->createDynamicMessage('Dimension ' . strtoupper($dimension) . ' subspace prohibited.', ['warning']);
			}
		}

		// Try getting messages and hints from database
		try {
			$result = $this->Messages->getMessagesAndHints($coordinates);

			$messages = array_merge(
				$messages,
				$result['messages']
			);

			$hints = $result['hints'];
		} catch (Exception $e) {
			$messages[] = $this->Messages->createDynamicMessage('Error: ' . $e->getMessage(), ['warning']);
		}

		// Set data for result rendering
		$this->set(compact('coordinates', 'messages', 'hints'));
		$this->set('_serialize', true);
	}
}
